@@ -25,8 +25,7 @@ Features
  * Supports http/https and SSL client-side certificates
  * Uses persistent HTTP connections by default
  * Properly converts to/from SOLR data types, including datetime objects
- * Supports both querying and update commands (add, delete).
- * Supports batching of commands
+ * Supports both querying and update commands (add, delete)
  * Requires Python 2.3+
 
 
@@ -96,7 +95,6 @@ Once created, a connection object has the following public methods:
                 add(id='foo', notes='bar')
 
             You must "commit" for the addition to be saved.
-            This command honors begin_batch/end_batch.
 
     add_many(lst)
 
@@ -108,66 +106,34 @@ Once created, a connection object has the following public methods:
                             {'id': 'foo2', 'notes': 'w00t'} ] )
 
             You must "commit" for the addition to be saved.
-            This command honors begin_batch/end_batch.
 
     delete(id)
 
             Delete a document by id.
 
             You must "commit" for the deletion to be saved.
-            This command honors begin_batch/end_batch.
 
     delete_many(lst)
 
             Delete a series of documents.  Pass in a list of ids.
 
             You must "commit" for the deletion to be saved.
-            This command honors begin_batch/end_batch.
 
     delete_query(query)
 
             Delete any documents returned by issuing a query.
 
             You must "commit" for the deletion to be saved.
-            This command honors begin_batch/end_batch.
 
 
     commit(wait_flush=True, wait_searcher=True)
 
             Issue a commit command.
 
-            This command honors begin_batch/end_batch.
-
     optimize(wait_flush=True, wait_searcher=True)
 
             Issue an optimize command.
 
-            This command honors begin_batch/end_batch.
-
-    begin_batch()
-
-            Begin "batch" mode, in which all commands to be sent
-            to the SOLR server are queued up and sent all at once.
-
-            No update commands will be sent to the backend server
-            until end_batch() is called. Not that "query" commands
-            are not batched.
-
-            begin_batch/end_batch transactions can be nested.
-            The transaction will not be sent to the backend server
-            until as many end_batch() calls have been made as
-            begin_batch()s.
-
-            Batching is completely optional. Any update commands
-            issued outside of a begin_batch()/end_batch() pair will
-            be immediately processed.
-
-    end_batch(commit=False)
-
-            End a batching pair.  Any pending commands are sent
-            to the backend server.  If "True" is passed in to
-            end_batch, a <commit> is also sent.
-
     raw_query(**params)
 
             Send a query command (unprocessed by this library) to
@@ -265,15 +231,6 @@ Examples showing the search wrapper
     >>> print response.results.start
      20
 
-Add 3 documents and delete 1, but send all of them as a single transaction.
-
-    >>> c.begin_batch()
-    >>> c.add(id="1")
-    >>> c.add(id="2")
-    >>> c.add(id="3")
-    >>> c.delete(id="0")
-    >>> c.end_batch(True)
-
 Enter a raw query, without processing the returned HTML contents.
 
     >>> print c.raw_query(q='id:[* TO *]', wt='python', rows='10')
@@ -286,13 +243,14 @@ import urlparse
 import codecs
 import urllib
 import datetime
+import logging
 from StringIO import StringIO
 from xml.sax import make_parser
 from xml.sax.handler import ContentHandler
 from xml.sax.saxutils import escape, quoteattr
 from xml.dom.minidom import parseString
 
-__version__ = "0.5"
+__version__ = "0.9.1"
 
 __all__ = ['SolrException', 'SolrConnection', 'Response']
 
@@ -333,7 +291,8 @@ class SolrConnection:
                  ssl_key=None,
                  ssl_cert=None,
                  post_headers={},
-                 max_retries=3):
+                 max_retries=3,
+                 debug=False):
 
         """
             url -- URI pointing to the SOLR instance. Examples:
@@ -381,7 +340,6 @@ class SolrConnection:
         else:
             self.conn = httplib.HTTPConnection(self.host, **kwargs)
 
-        self.batch_cnt = 0  #  this is int, not bool!
         self.response_version = 2.2
         self.encoder = codecs.getencoder('utf-8')
 
@@ -408,6 +366,8 @@ class SolrConnection:
         if not self.persistent:
             self.form_headers['Connection'] = 'close'
 
+        self.debug = debug
+
     def close(self):
         self.conn.close()
 
@@ -433,10 +393,11 @@ class SolrConnection:
         are given, raise a ValueError.
 
         sort is a list of fields to sort by. See "fields" for
-        formatting.
+        formatting. Each sort element can have be in the form
+        "fieldname asc|desc" as specified by SOLR specs.
 
-        sort_order defaults to "asc" and specifies the order to sort the fields
-        in. sort_order must be "asc" or "desc", otherwise a ValueError is raised.
+        sort_order is the backward compatible way to add the same ordering
+        to all the sort field when it is not specified.
 
         Optional parameters can also be passed in.  Many SOLR
         parameters are in a dotted notation (e.g., hl.simple.post).
@@ -476,9 +437,16 @@ class SolrConnection:
         if sort:
             if not sort_order or sort_order not in ("asc", "desc"):
                 raise ValueError("sort_order must be 'asc' or 'desc'")
-            if not isinstance(sort, basestring):
-                sort = ",".join(sort)
-            params['sort'] = "%s %s" % (sort, sort_order)
+            if isinstance(sort, basestring):
+                sort = [ f.strip() for f in sort.split(",") ]
+            sorting = []
+            for e in sort:
+                if not (e.endswith("asc") or e.endswith("desc")):
+                    sorting.append("%s %s" % (e, sort_order))
+                else:
+                    sorting.append(e)
+            sort = ",".join(sorting)
+            params['sort'] = sort
 
         if score and not 'score' in fields.replace(',',' ').split():
             fields += ',score'
@@ -489,6 +457,9 @@ class SolrConnection:
 
         request = urllib.urlencode(params, doseq=True)
 
+        if self.debug:
+            logging.info("solrpy request: %s" % request)
+
         try:
             rsp = self._post(self.path + '/select',
                               request, self.form_headers)
@@ -496,6 +467,9 @@ class SolrConnection:
             # and creating a StringIO, then Persistence breaks with
             # an internal python error.
             xml = StringIO(rsp.read())
+            if self.debug:
+                logging.info("solrpy got response: %s" % xml.getvalue())
+                
             data = parse_query_response(xml,  params=params, connection=self)
 
         finally:
@@ -504,51 +478,6 @@ class SolrConnection:
 
         return data
 
-    def begin_batch(self):
-        """
-        Denote the beginning of a batch update.
-
-        No update commands will be sent to the backend server
-        until end_batch() is called.
-
-        Any update commands issued outside of a begin_batch()/
-        end_batch() series will be immediately processed.
-
-        begin_batch/end_batch transactions can be nested.
-        The transaction will not be sent to the backend server
-        until as many end_batch() calls have been made as
-        begin_batch()s.
-        """
-        if not self.batch_cnt:
-            self.__batch_queue = []
-
-        self.batch_cnt += 1
-
-        return self.batch_cnt
-
-    def end_batch(self, commit=False):
-        """
-        Denote the end of a batch update.
-
-        Sends any queued commands to the backend server.
-
-        If `commit` is True, then a <commit/> command is included
-        at the end of the list of commands sent.
-        """
-        batch_cnt = self.batch_cnt - 1
-        if batch_cnt < 0:
-            raise SolrException(
-                "end_batch called without a corresponding begin_batch")
-
-        self.batch_cnt = batch_cnt
-        if batch_cnt:
-            return False
-
-        if commit:
-            self.__batch_queue.append('<commit/>')
-
-        return self._update("".join(self.__batch_queue))
-
     def delete(self, id):
         """
         Delete a specific document by id.
@@ -640,7 +569,7 @@ class SolrConnection:
         """
 
         # Clean up optional parameters to match SOLR spec.
-        params = dict([(key.replace('_','.'), unicode(value))
+        params = dict([(key.replace('_','.'), value)
                        for key, value in params.items()])
 
         request = urllib.urlencode(params, doseq=True)
@@ -657,11 +586,6 @@ class SolrConnection:
 
     def _update(self, request):
 
-        # If we're in batching mode, just queue up the requests for later.
-        if self.batch_cnt:
-            self.__batch_queue.append(request)
-            return
-
         try:
             rsp = self._post(self.path + '/update',
                               request, self.xmlheaders)
@@ -686,7 +610,7 @@ class SolrConnection:
         for field, value in fields.items():
             # Handle multi-valued fields if values
             # is passed in as a list/tuple
-            if not isinstance(value, (list, tuple)):
+            if not isinstance(value, (list, tuple, set)):
                 values = [value]
             else:
                 values = value
@@ -696,10 +620,12 @@ class SolrConnection:
                 if value == None:
                     continue
                 # Do some basic data conversion
-                if isinstance(value, datetime.date):
-                    value = datetime.datetime.combine(value, datetime.time(tzinfo=UTC()))
                 if isinstance(value, datetime.datetime):
                     value = utc_to_string(value)
+                elif isinstance(value, datetime.date):
+                    value = datetime.datetime.combine(
+                        value, datetime.time(tzinfo=UTC()))
+                    value = utc_to_string(value)
                 elif isinstance(value, bool):
                     value = value and 'true' or 'false'
 
@@ -766,6 +692,39 @@ class Response(object):
         self._connection = connection
         self._params = {}
 
+    def _set_numFound(self, value):
+        self._numFound = long(value)
+
+    def _get_numFound(self):
+        return self._numFound
+
+    def _del_numFound(self):
+        del self._numFound
+
+    numFound = property(_get_numFound, _set_numFound, _del_numFound)
+
+    def _set_start(self, value):
+        self._start = long(value)
+
+    def _get_start(self):
+        return self._start
+
+    def _del_start(self):
+        del self._start
+
+    start = property(_get_start, _set_start, _del_start)
+
+    def _set_maxScore(self, value):
+        self._maxScore = float(value)
+
+    def _get_maxScore(self):
+        return self._maxScore
+
+    def _del_maxScore(self):
+        del self._maxScore
+
+    maxScore = property(_get_maxScore, _set_maxScore, _del_maxScore)
+
     def __len__(self):
         """
         Return the number of matching documents contained in this set.
