@@ -18,6 +18,7 @@ from xml.dom.minidom import parseString
 
 # solrpy
 from solr import SolrConnection, SolrPaginator
+import solr.core
 
 SOLR_PATH = "/solr"
 SOLR_HOST = "localhost"
@@ -91,6 +92,38 @@ class TestAddingDocuments(unittest.TestCase):
         self.assertEquals(doc["user_id"], user_id)
         self.assertEquals(doc["data"], data)
 
+    def test_add_one_document_multiplefields(self):
+        """ Adds several documents with multiple fields, namely
+        a list, a tuple, and a set
+        """
+        user_id = get_rand_string()
+        data = get_rand_string()
+        letterset = [ u'a', u'b', u'c' ]
+        letters = [
+            letterset,
+            tuple(letterset),
+            set(letterset)
+        ]
+
+        for lset in letters:
+            doc = {}
+            doc["user_id"] = user_id
+            doc["data"] = data
+            doc["id"] = get_rand_string()
+            doc["letters"] = lset
+            self.conn.add(**doc)
+            self.conn.commit()
+            
+        results = self.conn.query("user_id:" + user_id).results
+
+        self.assertEquals(len(results), 3,
+            "Could not find expected data (user_id:%s)" % user_id)
+
+        for i, doc in enumerate(results):
+            self.assertEquals(doc["user_id"], user_id)
+            self.assertEquals(doc["data"], data)
+            self.assertEquals(doc["letters"], list(letters[i]))
+
     def test_add_one_document_implicit_commit(self):
         """ Try to add one document and commit changes in one operation.
         """
@@ -322,6 +355,90 @@ class TestAddingDocuments(unittest.TestCase):
         self.conn.close()
 
 
+class TestUpdatingDocuments(unittest.TestCase):
+
+    def setUp(self):
+        self.conn = SolrConnection(SOLR_HTTP)
+
+    def test_update_single(self):
+        """ Try to add one document, and then update it (readd it)
+        """
+        user_id = get_rand_string()
+        data = get_rand_string()
+        updated_data = get_rand_string()
+        id = get_rand_string()
+
+        doc = {}
+        doc["user_id"] = user_id
+        doc["data"] = data
+        doc["id"] = id
+
+        self.conn.add(**doc)
+        self.conn.commit()
+        
+        # we assume this works, being tested elsewhere
+        doc["data"] = updated_data
+        self.conn.add(**doc)
+        self.conn.commit()
+        
+        results = self.conn.query("id:" + id).results
+        doc = results[0]
+        self.assertEquals(doc["data"], updated_data)
+
+    def test_update_many(self):
+        """ Try to add more than one document in a single operation, and then
+        again to update them all in a single run.
+        """
+        doc_count = 10
+        user_ids = [get_rand_string() for x in range(doc_count)]
+        data =  [get_rand_string() for x in range(doc_count)]
+        updated_data = [get_rand_string() for x in range(doc_count)]
+        ids =  [get_rand_string() for x in range(doc_count)]
+        documents = [dict(user_id=user_ids[x], data=data[x], id=ids[x])
+                        for x in range(doc_count)]
+
+        self.conn.add_many(documents)
+        self.conn.commit()
+
+        # we assume the previous operation went through correctly, since
+        # it has a test all for itself.
+        for i, doc in enumerate(documents):
+            doc["data"] = updated_data[i]
+
+        self.conn.add_many(documents)
+        self.conn.commit()
+
+        results = []
+        for id in ids:
+            res = self.conn.query("id:" + id).results
+            if not res:
+                self.fail("Could not find document (id:%s)" % id)
+            results.append(res[0])
+
+        self.assertEquals(len(results), doc_count,
+            "Query didn't return all documents. Expected: %d, got: %d" % (
+                doc_count, len(results)))
+
+        query_user_ids = [doc["user_id"] for doc in results]
+        query_data = [doc["data"] for doc in results]
+        query_ids = [doc["id"] for doc in results]
+
+        # Symmetric difference will give us those documents which are neither
+        # in original list nor in a fetched one. It's a handy way to check
+        # whether all, and only those expected, documents have been returned.
+
+        user_ids_symdiff = set(user_ids) ^ set(query_user_ids)
+        data_symdiff = set(updated_data) ^ set(query_data)
+        ids_symdiff = set(ids) ^ set(query_ids)
+
+        self.assertEqual(user_ids_symdiff, set([]),
+            "User IDs sets differ (difference:%s)" % (user_ids_symdiff))
+        self.assertEqual(data_symdiff, set([]),
+            "Data sets differ (difference:%s)" % (data_symdiff))
+        self.assertEqual(ids_symdiff, set([]),
+            "IDs sets differ (difference:%s)" % (ids_symdiff))
+
+
 class TestDocumentsDeletion(unittest.TestCase):
 
     def setUp(self):
@@ -778,6 +895,51 @@ class TestQuerying(unittest.TestCase):
                 "Expected %s instead of %s on position %s in query_data:%s" % (
                     datum, query_data[idx], idx, query_data))
 
+    def test_query_sort_complex_sort_order(self):
+        """ Test whether sorting works (using non-default, descending, sort order).
+        """
+        doc_count = 10
+        prefix = get_rand_string()
+
+        data = [prefix + "-" + str(x) for x in range(5)]
+
+        # Two user ids
+        user_ids = [get_rand_string(), get_rand_string()]
+        # We sort 'em
+        user_ids.sort()
+
+        for user_id in user_ids:
+            for datum in data:
+                self.conn.add(id=get_rand_string(), user_id=user_id, data=datum)
+        self.conn.commit()
+
+        results = self.conn.query(
+            q="user_id:%s OR user_id:%s" % (user_ids[0], user_ids[1]),
+            sort=["user_id asc", "data desc"]).results
+
+        self.assertEquals(len(results), doc_count,
+            "There should be %d documents returned, got:%d, results:%s" % (
+                doc_count, len(results), results))
+
+        data.reverse()
+        # I'm not entirely sure wheter Python 2.3 supports this
+        # expected = [(a,b) for a in user_ids for b in data]
+        # If it does substitute to below
+        expected = []
+        for user_id in user_ids:
+            for d in data:
+                expected.append((user_id, d))
+
+        for idx, result in enumerate(results):
+            params =  (result['user_id'], result['data']) + expected[idx] + \
+                (idx, results, expected)
+            self.assertEquals(
+                (result['user_id'], result['data']),
+                expected[idx],
+                ("Expected %s, %s instead of %s, %s at position %s"
+                " in %s (expected %s)") % params
+            )
+
     def test_date(self):
         id = data = user_id = get_rand_string()
         date = datetime.date(1969, 5, 28)
@@ -790,6 +952,19 @@ class TestQuerying(unittest.TestCase):
         self.assertEqual(str(results[0]['creation_time']), 
                          '1969-05-28 00:00:00+00:00')
 
+    def test_datetime_utc(self):
+        id = data = user_id = get_rand_string()
+        dt = datetime.datetime(
+            1969, 5, 28, 12, 24, 42, tzinfo=solr.core.UTC())
+        self.conn.add(id=id, user_id=user_id, data=data, creation_time=dt)
+        self.conn.commit()
+        results = self.conn.query("id:%s" % id).results
+        self.assertEqual(len(results), 1)
+        self.assertTrue(isinstance(results[0]['creation_time'],
+                                   datetime.datetime))
+        self.assertEqual(str(results[0]['creation_time']), 
+                         '1969-05-28 12:24:42+00:00')
+
     def test_multi_date(self):
         id = data = user_id = get_rand_string()
         dates = [datetime.date(1969, 5, 28), datetime.date(2009, 1, 30)]
@@ -983,7 +1158,8 @@ class TestResponse(unittest.TestCase):
         self.conn = SolrConnection(SOLR_HTTP)
 
     def test_response_attributes(self):
-        """ Make sure Response objects have all the documented attributes.
+        """ Make sure Response objects have all the documented attributes,
+        and also checks that they are of the correct type
         """
         # Same id, data and user_id
         id = data = user_id = get_rand_string()
@@ -991,15 +1167,22 @@ class TestResponse(unittest.TestCase):
         self.conn.commit()
 
         response = self.conn.query(q="id:" + id)
-        expected_attrs = ["numFound", "start", "maxScore", "header"]
+        # here we also check the type of the attribute
+        expected_attrs = {
+            "numFound": long,
+            "start": long,
+            "maxScore": float,
+            "header": dict
+        }
 
-        for attr in expected_attrs:
-            self.assertTrue(attr in dir(response),
+        for attr, attr_type in expected_attrs.items():
+            self.assertTrue(hasattr(response, attr),
                 "Attribute %s not found in response. id:%s" % (attr, id))
 
             value = getattr(response, attr)
-            self.assertTrue(bool(value),
-                "Attribute %s has no value. id:%s" % (attr,id))
+            # check type
+            self.assertTrue(isinstance(value, attr_type),
+                "Attribute %s has wrong type. id:%s" % (attr,id))
 
     def tearDown(self):
         self.conn.close()
